SystemOrganization addCategory: #'CompressionTests-Archive'!

TestCase subclass: #ZipArchiveTest
	instanceVariableNames: 'fileToZip zip subdir zipFile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CompressionTests-Archive'!

!ZipArchiveTest methodsFor: 'tests' stamp: 'SeanDeNigris 5/19/2012 23:33'!
setUp

	| now |
	now := DateAndTime now asString replaceAll: $: with: $_.
	fileToZip := ('_test-zip-file', now) asFileReference.
	fileToZip writeStreamDo: [ :stream | stream nextPutAll: 'file contents' ].
	subdir := ('_test-zip-dir', now) asFileReference.
	subdir ensureDirectory.
	zipFile := ('pharo_test', now) asFile, 'zip'.
	zip := ZipArchive new.
! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'SeanDeNigris 5/18/2012 23:24'!
tearDown

	fileToZip delete.
	zip close.	
	subdir deleteAll.
	zipFile delete.! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'SeanDeNigris 5/18/2012 22:31'!
testAddNonExistentFile
	
	self should: [ zip addFile: 'it_would_be_crazy_if_this_file_existed.ext' ] raise: FileDoesNotExistException.! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'SeanDeNigris 5/20/2012 09:47'!
testCreateWithRelativeNames
	"Test creating a zip with a relative tree of files, so that the tree will
	be created whereever the ."
	
	| nestedFileToZip localFileHeaderSignature versionNeededToExtract bitFlag deflateCompressionMethod expectedCrc32 expectedUncompressedSize folderModified centralDirectoryOffset centralDirectoryEnd expectedCompressedSize fileModified fileStart |
	nestedFileToZip := subdir / '_test-zip-file'.
	nestedFileToZip writeStreamDo: [ :stream | stream nextPutAll: 'file contents' ].
	fileModified := nestedFileToZip entry modification asTime.
	folderModified := subdir entry modification asTime.
	zip
		addDirectory: subdir fullName
		as: subdir name.
	zip	
		addFile: nestedFileToZip fullName
		as: nestedFileToZip name.
	zip writeToFileNamed: zipFile fullName.
	
zipFile fileStreamDo: [ :str |
		str binary.
		localFileHeaderSignature := 16r04034b50.
		versionNeededToExtract := 20.
		bitFlag := 0.
		deflateCompressionMethod := 8.
		expectedCrc32 := 16r2ab092ee. "I don''t know how to compute this, but OS X returned the same, so guessing it's correct"
		expectedCompressedSize := 15.
		expectedUncompressedSize := 13.
		
		"folder"
		self assert: (str nextLittleEndianNumber: 4) equals: localFileHeaderSignature.
		self assert: (str nextLittleEndianNumber: 2) equals: versionNeededToExtract.
		self assert: (str nextLittleEndianNumber: 2) equals: bitFlag.
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "No compression".
		self assert: ((str nextLittleEndianNumber: 2) asTimeFromZip between: (folderModified addSeconds: -1) and: (folderModified addSeconds: 1)).
		self assert: (str nextLittleEndianNumber: 2) asDateFromZip equals: subdir entry modification asDate.
		self assert: (str nextLittleEndianNumber: 4) equals: 0 "expectedCrc32".
		self assert: (str nextLittleEndianNumber: 4) equals: 0 "expectedCompressedSize".
		self assert: (str nextLittleEndianNumber: 4) equals: 0 "expectedUncompressedSize".
		self assert: (str nextLittleEndianNumber: 2) equals: subdir basename size + 1.
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Extra header length".
		self assert: (str next: subdir basename size + 1) asString equals: subdir basename, '/'.
		
		"Test file"
		fileStart := str position.
		self assert: (str nextLittleEndianNumber: 4) equals: localFileHeaderSignature.
		self assert: (str nextLittleEndianNumber: 2) equals: versionNeededToExtract.
		self assert: (str nextLittleEndianNumber: 2) equals: bitFlag.
		self assert: (str nextLittleEndianNumber: 2) equals: deflateCompressionMethod.
		self assert: ((str nextLittleEndianNumber: 2) asTimeFromZip between: (fileModified addSeconds: -1) and: (fileModified addSeconds: 1)).
		self assert: (str nextLittleEndianNumber: 2) asDateFromZip equals: nestedFileToZip entry modification asDate.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedCrc32.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedCompressedSize.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedUncompressedSize.
		self assert: (str nextLittleEndianNumber: 2) equals: nestedFileToZip name size.
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Extra header length".
		self assert: (str next: nestedFileToZip name size) asString equals: nestedFileToZip name.
		self assert: (str next: expectedCompressedSize) isNil not "I don''t understand the compression yet".
		
		"Central directory structure"
		centralDirectoryOffset := str position.
		
		"Folder entry"
		self assert: (str nextLittleEndianNumber: 4) equals: 16r02014b50.	
		self flag: 'I think this is wrong. What is version 1.4?!!'.
		self assert: (str nextLittleEndianNumber: 2) equals:  16r314.
		self assert: (str nextLittleEndianNumber: 2) equals: versionNeededToExtract.
		self assert: (str nextLittleEndianNumber: 2) equals: bitFlag.
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "no compression".
		self assert: ((str nextLittleEndianNumber: 2) asTimeFromZip between: (folderModified addSeconds: -1) and: (folderModified addSeconds: 1)).
		self assert: (str nextLittleEndianNumber: 2) asDateFromZip equals: fileToZip entry modification asDate.
		self assert: (str nextLittleEndianNumber: 4) equals: 0 "expectedCrc32".
		self assert: (str nextLittleEndianNumber: 4) equals: 0 "expectedCompressedSize".
		self assert: (str nextLittleEndianNumber: 4) equals: 0 "expectedUncompressedSize".
		self assert: (str nextLittleEndianNumber: 2) equals: subdir basename size + 1.
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Extra header length".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "File comment".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Disk number start".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Internal file attributes".
		
		self flag: 'I don''t understand the attributes yet'.
		self assert: (str nextLittleEndianNumber: 4) equals:  8r040755 << 16 "External file attributes".
		self assert: (str nextLittleEndianNumber: 4) equals:  0 "Relative offset of local header".
		self assert: (str next: subdir basename size + 1) asString equals: subdir basename, '/'.
		
		"Nested file entry"
		self assert: (str nextLittleEndianNumber: 4) equals: 16r02014b50.
		
		self flag: 'I think this is wrong. What is version 1.4?!!'.
		self assert: (str nextLittleEndianNumber: 2) equals:  16r314.
		self assert: (str nextLittleEndianNumber: 2) equals: versionNeededToExtract.
		self assert: (str nextLittleEndianNumber: 2) equals: bitFlag.
		self assert: (str nextLittleEndianNumber: 2) equals: deflateCompressionMethod.
		self assert: ((str nextLittleEndianNumber: 2) asTimeFromZip between: (fileModified addSeconds: -1) and: (fileModified addSeconds: 1)).
		self assert: (str nextLittleEndianNumber: 2) asDateFromZip equals: nestedFileToZip entry modification asDate.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedCrc32.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedCompressedSize.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedUncompressedSize.
		self assert: (str nextLittleEndianNumber: 2) equals: nestedFileToZip name size.
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Extra header length".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "File comment".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Disk number start".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Internal file attributes".
		
		self flag: 'I don''t understand the attributes yet'.
		self assert: (str nextLittleEndianNumber: 4) equals: 8r0100666 << 16 "External file attributes".
		self assert: (str nextLittleEndianNumber: 4) equals: fileStart "Relative offset of local header".
		self assert: (str next: nestedFileToZip name size) asString equals: nestedFileToZip name.

		centralDirectoryEnd := str position.
		
		"End of central directory record"
		self assert: (str nextLittleEndianNumber: 4) equals: 16r6054B50 "Signature".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Disk number".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Disk # where central dir started".
		self assert: (str nextLittleEndianNumber: 2) equals: 2 "Total entries in central dir on this disk".
		self assert: (str nextLittleEndianNumber: 2) equals: 2 "Total entries in central dir".
		self assert: (str nextLittleEndianNumber: 4) equals: centralDirectoryEnd  - centralDirectoryOffset "Central directory size".
		self assert: (str nextLittleEndianNumber: 4) equals: centralDirectoryOffset "from start of first disk".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "zip comment length".
		self assert: str atEnd ].! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'CamilloBruni 6/17/2012 17:14'!
testDate29Feb2000
	"Ensure that dates with leap years don't screw up in the conversion"

	| archive member theDate |
	theDate := Date year: 2000 month: 2 day: 29.
	archive := ZipArchive new.
	member := archive addDeflateString: 'foo' as: 'bar'.
	member modifiedAt: theDate.
	self assert: member lastModTime = theDate asDateAndTime.! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'SeanDeNigris 5/20/2012 08:55'!
testFilePermissions

	| nestedFileToZip fileModified folderModified folderMember fileMember |
	nestedFileToZip := subdir / '_test-zip-file'.
	nestedFileToZip writeStreamDo: [ :stream | stream nextPutAll: 'file contents' ].
	fileModified := nestedFileToZip entry modification asTime.
	folderModified := subdir entry modification asTime.
	folderMember := zip
		addDirectory: subdir fullName
		as: subdir name.
	fileMember := zip	
		addFile: nestedFileToZip fullName
		as: nestedFileToZip name.
	zip writeToFileNamed: zipFile fullName.
	
	self assert: fileMember unixFileAttributes equals: 8r100666.
	self assert: folderMember unixFileAttributes equals: 8r40755.! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'SeanDeNigris 5/18/2012 22:37'!
testPackDate

	self assert: '16 May 2012' asDate zipped equals: 16r40b0.! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'SeanDeNigris 5/18/2012 22:37'!
testPackTime

	self assert: '16:16:56' asTime zipped equals: 16r821C.
	self assert: '1:11:54pm' asTime zipped equals: 16r697b.! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'CamilloBruni 6/17/2012 17:14'!
testSetLastModification

	| theDate member |
	theDate := Date year: 2000 month: 1 day: 15.
	member := zip addDeflateString: 'foo' as: 'bar'.
	member modifiedAt: theDate.
	self assert: member lastModTime equals: theDate asDateAndTime.! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'SeanDeNigris 5/20/2012 10:08'!
testUnpackDate
	
	self assert: 16r40b0 asDateFromZip equals: '16 May 2012' asDate.! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'SeanDeNigris 5/20/2012 10:11'!
testUnpackTime

	self assert: 16r821C asTimeFromZip equals: '16:16:56' asTime.! !

!ZipArchiveTest methodsFor: 'tests' stamp: 'SeanDeNigris 5/18/2012 23:25'!
testZip
	
	| localFileHeaderSignature versionNeededToExtract bitFlag deflateCompressionMethod realModificationTime expectedCrc32 expectedCompressedSize expectedUncompressedSize centralDirectoryOffset centralDirectoryEnd |
	realModificationTime := fileToZip entry modification asTime.
	zip	
		addFile: fileToZip fullName
		as: fileToZip name.
	zip writeToFileNamed: zipFile fullName.
	zipFile fileStreamDo: [ :str |
		str binary.
		localFileHeaderSignature := 16r04034b50.
		versionNeededToExtract := 20.
		bitFlag := 0.
		deflateCompressionMethod := 8.
		expectedCrc32 := 16r2ab092ee. "I don''t know how to compute this, but OS X returned the same, so guessing it's correct"
		expectedCompressedSize := 15.
		expectedUncompressedSize := 13.
		
		"Test file"
		self assert: (str nextLittleEndianNumber: 4) equals: localFileHeaderSignature.
		self assert: (str nextLittleEndianNumber: 2) equals: versionNeededToExtract.
		self assert: (str nextLittleEndianNumber: 2) equals: bitFlag.
		self assert: (str nextLittleEndianNumber: 2) equals: deflateCompressionMethod.
		self assert: ((str nextLittleEndianNumber: 2) asTimeFromZip between: (realModificationTime addSeconds: -1) and: (realModificationTime addSeconds: 1)).
		self assert: (str nextLittleEndianNumber: 2) asDateFromZip equals: fileToZip entry modification asDate.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedCrc32.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedCompressedSize.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedUncompressedSize.
		self assert: (str nextLittleEndianNumber: 2) equals: fileToZip basename size.
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Extra header length".
		self assert: (str next: fileToZip basename size) asString equals: fileToZip basename.
		self assert: (str next: expectedCompressedSize) isNil not "I don''t understand the compression yet".
		
		"Central directory structure"
		centralDirectoryOffset := str position.
		self assert: (str nextLittleEndianNumber: 4) equals: 16r02014b50.
		
		self flag: 'I think this is wrong. What is version 1.4?!!'.
		self assert: (str nextLittleEndianNumber: 2) equals:  16r314.
		self assert: (str nextLittleEndianNumber: 2) equals: versionNeededToExtract.
		self assert: (str nextLittleEndianNumber: 2) equals: bitFlag.
		self assert: (str nextLittleEndianNumber: 2) equals: deflateCompressionMethod.
		self assert: ((str nextLittleEndianNumber: 2) asTimeFromZip between: (realModificationTime addSeconds: -1) and: (realModificationTime addSeconds: 1)).
		self assert: (str nextLittleEndianNumber: 2) asDateFromZip equals: fileToZip entry modification asDate.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedCrc32.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedCompressedSize.
		self assert: (str nextLittleEndianNumber: 4) equals: expectedUncompressedSize.
		self assert: (str nextLittleEndianNumber: 2) equals: fileToZip basename size.
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Extra header length".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "File comment".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Disk number start".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Internal file attributes".
		
		self flag: 'I don''t understand the attributes yet'.
		self assert: (str nextLittleEndianNumber: 4) equals:  16r81B60000 "External file attributes".
		self assert: (str nextLittleEndianNumber: 4) equals:  0 "Relative offset of local header".
		self assert: (str next: fileToZip basename size) asString equals: fileToZip basename.
		centralDirectoryEnd := str position.
		
		"End of central directory record"
		self assert: (str nextLittleEndianNumber: 4) equals: 16r6054B50 "Signature".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Disk number".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "Disk # where central dir started".
		self assert: (str nextLittleEndianNumber: 2) equals: 1 "Total entries in central dir on this disk".
		self assert: (str nextLittleEndianNumber: 2) equals: 1 "Total entries in central dir".
		self assert: (str nextLittleEndianNumber: 4) equals: centralDirectoryEnd  - centralDirectoryOffset "Central directory size".
		self assert: (str nextLittleEndianNumber: 4) equals: centralDirectoryOffset "from start of first disk".
		self assert: (str nextLittleEndianNumber: 2) equals: 0 "zip comment length".
		self assert: str atEnd ].! !
